﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Management;
using Microsoft.Win32;
using System.Drawing;

namespace TweakAPI.Info
{
    /// <summary>
    /// Provides information about the processor.
    /// </summary>
    public static class Processor
    {
        private static ManagementObjectSearcher searcher = new ManagementObjectSearcher("root\\CIMV2", "SELECT * FROM Win32_Processor");

        /// <summary>
        /// Returns the name of the processor.
        /// </summary>
        public static string Name
        {
            get
            {
                RegistryKey regKey = Registry.LocalMachine.OpenSubKey("HARDWARE\\DESCRIPTION\\System\\CentralProcessor\\0", 
                    RegistryKeyPermissionCheck.ReadSubTree);
                return regKey.GetValue("ProcessorNameString").ToString();
            }
        }

        /// <summary>
        /// Returns the ID of the processor.
        /// </summary>
        public static string ID
        {
            get
            {
                RegistryKey regKey = Registry.LocalMachine.OpenSubKey("HARDWARE\\DESCRIPTION\\System\\CentralProcessor\\0", 
                    RegistryKeyPermissionCheck.ReadSubTree);
                return regKey.GetValue("Identifier").ToString();
            }
        }

        /// <summary>
        /// Returns the vendor ID of the processor.
        /// </summary>
        public static string VendorID
        {
            get
            {
                RegistryKey regKey = Registry.LocalMachine.OpenSubKey("HARDWARE\\DESCRIPTION\\System\\CentralProcessor\\0", 
                    RegistryKeyPermissionCheck.ReadSubTree);
                return regKey.GetValue("VendorIdentifier").ToString();
            }
        }

        /// <summary>
        /// Returns the clock speed of the processor.
        /// </summary>
        public static string ClockSpeed
        {
            get
            {
                RegistryKey regKey = Registry.LocalMachine.OpenSubKey("HARDWARE\\DESCRIPTION\\System\\CentralProcessor\\0", 
                    RegistryKeyPermissionCheck.ReadSubTree);
                string clockSpeedMhz = regKey.GetValue("~MHz").ToString();
                return Math.Round(Convert.ToDecimal(clockSpeedMhz) / 1000) + " GHz (" + clockSpeedMhz + " MHz)";
            }
        }

        /// <summary>
        /// Returns the address width of the processor.
        /// </summary>
        public static int AddressWidth
        {
            get
            {
                int addressWidth = 0;
                foreach (ManagementObject queryObj in searcher.Get())
                {
                    addressWidth = int.Parse(queryObj["AddressWidth"].ToString());
                }
                return addressWidth;
            }
        }

        /// <summary>
        /// Returns the architecture of the processor.
        /// </summary>
        public static Helper.Architecture Architecture
        {
            get
            {
                Helper.Architecture architecture = Helper.Architecture.x64;

                int num = AddressWidth;
                if (num == 64)
                { architecture = Helper.Architecture.x64; }
                else if (num == 32)
                { architecture = Helper.Architecture.x86; }

                return architecture;
            }
        }

        /// <summary>
        /// Returns the status of the processor.
        /// </summary>
        public static string CPUStatus
        {
            get
            {
                string status = null;
                foreach (ManagementObject queryObj in searcher.Get())
                {
                    status = queryObj["CpuStatus"].ToString();
                }
                return status;
            }
        }

        /// <summary>
        /// Returns the data width of the processor.
        /// </summary>
        public static int DataWidth
        {
            get
            {
                int dataWidth = 0;
                foreach (ManagementObject queryObj in searcher.Get())
                {
                    dataWidth = Convert.ToInt32(queryObj["DataWidth"].ToString());
                }
                return dataWidth;
            }
        }

        /// <summary>
        /// Returns the device ID of the processor.
        /// </summary>
        public static string DeviceID
        {
            get
            {
                string deviceID = null;
                foreach (ManagementObject queryObj in searcher.Get())
                {
                    deviceID = queryObj["DeviceID"].ToString();
                }
                return deviceID;
             }
        }

        /// <summary>
        /// Returns the external clock speed of the processor.
        /// </summary>
        public static string ExternalClockSpeed
        {
            get
            {
                string extClockSpeed = null;
                foreach (ManagementObject queryObj in searcher.Get())
                {
                    extClockSpeed = queryObj["ExtClock"].ToString() + " MHz";
                }
                return extClockSpeed;
            }
        }

        /// <summary>
        /// Returns the device family of the processor.
        /// </summary>
        public static string Family
        {
            get
            {
                string family = null;
                foreach (ManagementObject queryObj in searcher.Get())
                {
                    family = queryObj["Family"].ToString();
                }
                return family;
            }
        }

        /// <summary>
        /// Returns the L2 cache size of the processor.
        /// </summary>
        public static string L2CacheSize
        {
            get
            {
                string l2CacheSize = null;
                foreach (ManagementObject queryObj in searcher.Get())
                {
                    l2CacheSize = queryObj["L2CacheSize"].ToString() + " KiB";
                }
                return l2CacheSize;
            }
        }

        /// <summary>
        /// Returns the L3 cache size of the processor.
        /// </summary>
        public static string L3CacheSize
        {
            get
            {
                string l3CacheSize = null;
                foreach (ManagementObject queryObj in searcher.Get())
                {
                    l3CacheSize = queryObj["L3CacheSize"].ToString() + " KiB";
                }
                return l3CacheSize;
            }
        }

        /// <summary>
        /// Returns the level of the processor.
        /// </summary>
        public static int Level
        {
            get
            {
                int level = 0;
                foreach (ManagementObject queryObj in searcher.Get())
                {
                    level = Convert.ToInt32(queryObj["Level"].ToString());
                }
                return level;
            }
        }

        /// <summary>
        /// Returns the load percentage of the processor.
        /// </summary>
        public static int LoadPercentage
        {
            get
            {
                int loadPercentage = 0;
                foreach (ManagementObject queryObj in searcher.Get())
                {
                    loadPercentage = Convert.ToInt32(queryObj["LoadPercentage"].ToString());
                }
                return loadPercentage;
            }
        }

        /// <summary>
        /// Returns the number of cores.
        /// </summary>
        public static int Cores
        {
            get
            {
                int cores = 0;
                foreach (ManagementObject queryObj in searcher.Get())
                {
                    cores = Convert.ToInt32(queryObj["NumberOfCores"].ToString());
                }
                return cores;
            }
        }
        /// <summary>
        /// Returns the number of logical processors.
        /// </summary>
        public static int LogicalProcessors
        {
            get
            {
                return Environment.ProcessorCount;
            }
        }

        /// <summary>
        /// Returns the type of the processor.
        /// </summary>
        public static int ProcessorType
        {
            get
            {
                int processorType = 0;
                foreach (ManagementObject queryObj in searcher.Get())
                {
                    processorType = Convert.ToInt32(queryObj["ProcessorType"].ToString());
                }
                return processorType;
            }
        }

        /// <summary>
        /// Returns, whether power management is supported by the processor.
        /// </summary>
        public static bool PowerManagementSupported
        {
            get
            {
                bool powerManageSupport = false;
                foreach (ManagementObject queryObj in searcher.Get())
                {
                    powerManageSupport = Convert.ToBoolean(queryObj["PowerManagementSupported"].ToString());
                }
                return powerManageSupport;
            }
        }

        /// <summary>
        /// Returns the revision of the processor.
        /// </summary>
        public static string Revision
        {
            get
            {
                string revision = null;
                foreach (ManagementObject queryObj in searcher.Get())
                {
                    revision = queryObj["Revision"].ToString();
                }
                return revision;
            }
        }

        /// <summary>
        /// Returns the role of the processor.
        /// </summary>
        public static string Role
        {
            get
            {
                string role = null;
                foreach (ManagementObject queryObj in searcher.Get())
                {
                    role = queryObj["Role"].ToString();
                }
                return role;
            }
        }

        /// <summary>
        /// Returns the socket designation of the processor.
        /// </summary>
        public static string SocketDesignation
        {
            get
            {
                string socketDesignation = null;
                foreach (ManagementObject queryObj in searcher.Get())
                {
                    socketDesignation = queryObj["SocketDesignation"].ToString();
                }
                return socketDesignation;
            }
        }

        /// <summary>
        /// Returns the stepping of the processor.
        /// </summary>
        public static string Stepping
        {
            get
            {
                string stepping = null;
                foreach (ManagementObject queryObj in searcher.Get())
                {
                    stepping = queryObj["Stepping"].ToString();
                }
                return stepping;
            }
        }

        /// <summary>
        /// Returns the system name of the processor.
        /// </summary>
        public static string SystemName
        {
            get
            {
                string sysName = null;
                foreach (ManagementObject queryObj in searcher.Get())
                {
                    sysName = queryObj["SystemName"].ToString();
                }
                return sysName;
            }
        }

        /// <summary>
        /// Returns the unique ID of the processor.
        /// </summary>
        public static string UniqueID
        {
            get
            {
                string uniqueID = null;
                foreach (ManagementObject queryObj in searcher.Get())
                {
                    uniqueID = queryObj["UniqueId"].ToString();
                }
                return uniqueID;
            }
        }

        /// <summary>
        /// Returns the upgrade method of the processor.
        /// </summary>
        public static int UpgradeMethod
        {
            get
            {
                int upgradeMethod = 0;
                foreach (ManagementObject queryObj in searcher.Get())
                {
                    upgradeMethod = Convert.ToInt32(queryObj["UpgradeMethod"].ToString());
                }
                return upgradeMethod;
            }
        }

        /// <summary>
        /// Returns, whether the virtualization firmware is enabled.
        /// </summary>
        public static bool VirtualizationFirmwareEnabled
        {
            get
            {
                bool virtFirmwareEnabled = false;
                foreach (ManagementObject queryObj in searcher.Get())
                {
                    virtFirmwareEnabled = Convert.ToBoolean(queryObj["VirtualizationFirmwareEnabled"].ToString());
                }
                return virtFirmwareEnabled;
            }
        }

        /// <summary>
        /// Returns, whether VM monitor mode extensions are enabled.
        /// </summary>
        public static bool VMMonitorModeExtensions
        {
            get
            {
                bool vmMonModeExts = false;
                foreach (ManagementObject queryObj in searcher.Get())
                {
                    vmMonModeExts = Convert.ToBoolean(queryObj["VMMonitorModeExtensions"].ToString());
                }
                return vmMonModeExts;
            }
        }

        /// <summary>
        /// Returns the temperature of the processor.
        /// </summary>
        public static string[] Temperature
        {
            get
            {
                List<string> temps = new List<string>();
                ManagementObjectSearcher tempSearcher = new ManagementObjectSearcher(@"root\WMI", "SELECT * FROM MSAcpi_ThermalZoneTemperature");
                try
                {
                    foreach (ManagementObject tempSensor in tempSearcher.Get())
                    {
                        double temp = (double.Parse(tempSensor["CurrentTemperature"].ToString()) - 2731.5) / 10.0;
                        string temperature = temp.ToString() + " °C (" + (temp + 273.15).ToString() + " K; " +
                            (temp * 1.8 + 32.0).ToString() + " °F)";
                        temps.Add(temperature);
                    }
                }
                catch { temps.Add("Nicht unterstützt"); }
                return temps.ToArray();
            }
        }
    }
}
